@@ -1,5 +1,5 @@
-dat <- readRDS('previous.rds')
-png('mygraph.png')
+dat <- readRDS("previous.rds")
+png("mygraph.png")
 par(mar = c(15, 4, .5, .5))
 barplot(setNames(dat$number, dat$name), las = 2)
 dev.off()
