@@ -169,7 +169,7 @@ test_that("parse messages", {
 
 test_that("append changelog", {
   skip_on_cran_windows()
-  path <- prepare_orderly_example("changelog", testing = TRUE)
+  path <- test_prepare_orderly_example("changelog", testing = TRUE)
 
   tmp <- tempfile()
   path_example <- file.path(path, "src", "example")
@@ -221,7 +221,7 @@ test_that("append changelog", {
 
 test_that("label change requires rebuild", {
   skip_on_cran_windows()
-  path <- prepare_orderly_example("changelog", testing = TRUE)
+  path <- test_prepare_orderly_example("changelog", testing = TRUE)
 
   tmp <- tempfile()
   path_example <- file.path(path, "src", "example")
@@ -248,7 +248,7 @@ test_that("label change requires rebuild", {
 
 
 test_that("label values are checked", {
-  path <- prepare_orderly_example("changelog", testing = TRUE)
+  path <- test_prepare_orderly_example("changelog", testing = TRUE)
 
   tmp <- tempfile()
   path_example <- file.path(path, "src", "example")
@@ -262,7 +262,7 @@ test_that("label values are checked", {
 
 
 test_that("reports can't use changelogs if not enabled", {
-  path <- prepare_orderly_example("minimal")
+  path <- test_prepare_orderly_example("minimal")
   tmp <- tempfile()
   path_example <- file.path(path, "src", "example")
   path_cl <- path_changelog_txt(path_example)
@@ -276,7 +276,7 @@ test_that("reports can't use changelogs if not enabled", {
 
 test_that("public changelog", {
   skip_on_cran_windows()
-  path <- prepare_orderly_example("changelog", testing = TRUE)
+  path <- test_prepare_orderly_example("changelog", testing = TRUE)
 
   name <- "example"
   ids <- character(10)
@@ -318,7 +318,7 @@ test_that("public changelog", {
 
 test_that("public changelog with multiple entries", {
   skip_on_cran_windows()
-  path <- prepare_orderly_example("changelog", testing = TRUE)
+  path <- test_prepare_orderly_example("changelog", testing = TRUE)
 
   name <- "example"
   ids <- character(3)
@@ -360,7 +360,7 @@ test_that("public changelog with multiple entries", {
 
 test_that("public version has no changelog", {
   skip_on_cran_windows()
-  path <- prepare_orderly_example("changelog", testing = TRUE)
+  path <- test_prepare_orderly_example("changelog", testing = TRUE)
 
   name <- "example"
   ids <- character(4)
